#ifndef _BIP_Engine_Compound_HPP_
#define _BIP_Engine_Compound_HPP_

// inherited classes
#include <CompoundItf.hpp>
#include "Component.hpp"
#include "CompoundExportPort.hpp"
#include "Connector.hpp"
#include "Priority.hpp"

class InteractionValue;
class AtomInternalPort;
class BipError;

class Compound : public virtual ComponentItf, public Component, public CompoundItf {
 public:
  // constructors
  Compound(const string &name);

  // destructor
  virtual ~Compound();

  // specific
  virtual BipError &execute(PortValue &portValue);
  BipError &execute(InteractionValue &interactionValue);
  virtual BipError &initialize();
  BipError &update();
  void resetAllAtoms();
  BipError &recursiveUpdate();

  const vector<Connector *> &nonExportedConnectors() const;
  const vector<AtomInternalPort *> &nonExportedInternalPorts() const;

 protected:
  // protected setters for references
  void addComponent(Component &component) {
    mComponents[component.name()] = &component;
    component.setHolder(*this);
  }

  void addPort(CompoundExportPort &port) {
    mPorts[port.name()] = &port;
    port.setHolder(*this);
    Component::addPort(port);
  }

  void addPriority(Priority &priority) {
    mPriorities.push_back(&priority);
    priority.setHolder(*this);
  }

  void addConnector(Connector &connector) {
    mConnectors[connector.name()] = &connector;
    connector.setHolder(*this);
  }

  // specific
  BipError &detectCycleInPriorities() const;
  void updatePortValuesOfConnectors();
  void updatePortValuesOfExportedPorts();

  void computeNonExportedConnectors() const;
  void computeNonExportedInternalPorts() const;

  mutable bool mResetInteractions;
  mutable vector<InteractionValue *> mInteractions;

  mutable bool mResetInternals;
  mutable vector<AtomInternalPort *> mInternals;

  mutable bool mInitializedNonExportedConnectors;
  mutable vector<Connector *> mNonExportedConnectors;

  mutable bool mInitializedNonExportedInternalPorts;
  mutable vector<AtomInternalPort *> mNonExportedInternalPorts;

  // specific rights for deploying the system
  friend Component* deploy(int argc, char** argv);
};

inline const vector<Connector *> &Compound::nonExportedConnectors() const {
  if (!mInitializedNonExportedConnectors) {
    computeNonExportedConnectors();
  }

  return mNonExportedConnectors;
}

inline const vector<AtomInternalPort *> &Compound::nonExportedInternalPorts() const {
  if (!mInitializedNonExportedInternalPorts) {
    computeNonExportedInternalPorts();
  }

  return mNonExportedInternalPorts;
}

#endif // _BIP_Engine_Compound_HPP_
