#ifndef _BIP_Engine_CompoundExportPort_HPP_
#define _BIP_Engine_CompoundExportPort_HPP_

// inherited classes
#include <CompoundExportPortItf.hpp>
#include "Port.hpp"
#include "PortValue.hpp"

class BipError;

class CompoundExportPort : public virtual PortItf, public virtual Port, public CompoundExportPortItf {
 public:
  // constructors
  CompoundExportPort(const string &name);

  // destructor
  virtual ~CompoundExportPort();

  // specific
  virtual BipError &execute(PortValue& portValue);
  void updatePortValues();

  bool hasPortValue(PortValue& portValue) const;

  virtual vector<PortValue *> &portValues() { return mPortValues; }
  virtual const vector<PortValue *> &portValues() const { return mPortValues; }
  virtual bool hasPortValues() const { return !mPortValues.empty(); }

  virtual void addPortValue(PortValue &portValue) { mPortValues.push_back(&portValue); }
  virtual void clearPortValues() { mPortValues.clear(); }

  bool isReset() const { return mIsReset; }
  virtual void reset();

 protected:
  void addForwardPort(Port &forwardPort);


  vector<PortValue *> mPortValues;

  /**
   * \brief Associate a port to a port value.
   */
  void mapPortValue(PortValue &portValue, Port &port) {
    // if it's a final destination, map it!
    if (port.type() == ATOM_EXPORT ||
        port.type() == CONNECTOR_EXPORT) {
      portValue.setPort(port);
    }
  }

  bool mIsReset;


  // specific rights for deploying the system
  friend Component* deploy(int argc, char** argv);
};

#endif // _BIP_Engine_CompoundExportPort_HPP_
