#ifndef _BIP_Engine_Connector_HPP_
#define _BIP_Engine_Connector_HPP_

// inherited classes
#include <ConnectorItf.hpp>
#include "ConnectorExportPort.hpp"
#include "QuotedPortReference.hpp"

class Connector : public ConnectorItf {
 public:
  // constructors
  Connector(const string &name);

  // destructor
  virtual ~Connector();

  // operations
  virtual const vector<Interaction *> &interactions() const;

  // specific
  bool isTopLevel() const;
  const vector<Connector *> &subConnectors() const;
  const vector<Connector *> &allSubConnectors() const;
  void updatePortValues();
  const vector<InteractionValue *> &enabledInteractions() const;
  const vector<InteractionValue *> &maximalInteractions() const;
  const vector<InteractionValue *> &locallyMaximalInteractions() const;
  void release() const;
  void release(const vector<Interaction *> &interactions) const;
  void release(const vector<InteractionValue *> &interactions) const;

  const vector<Priority *> &dominatingPriorities() const;
  const vector<Priority *> &dominatedPriorities() const;
  
  void reset() const;
  bool isReset() const { return mIsReset; }
  void resetMaximal() const;
  void resetEnabledPorts() const { mResetAllEnabledPorts = true; };

  string fullName() const;

 protected:
  // protected setters for references
  void addPort(QuotedPortReference &port) {
    mPorts.push_back(&port);
    port.setHolder(*this);
    port.port().addConnectedConnector(*this);
  }

  void setExportedPort(ConnectorExportPort &exportedPort) {
    mExportedPort = &exportedPort;
    exportedPort.setHolder(*this);
  }

  // specific
  void enumerateInteractions(vector<Interaction *> &allInteractions, vector<QuotedPortReference *> &partialInteraction, unsigned int nextPortIndex) const;
  void enumerateInteractionValues(vector<InteractionValue *> &allInteractions, const Interaction &interaction, vector<PortValue *> &partialValues, unsigned int nextPortIndex, bool keepLocallyMaximalOnly) const;
  void enumerateLocallyMaximalInteractionValues(vector<InteractionValue *> &allInteractions, Interaction &interaction, unsigned int mandatoryIndex) const;

  bool resetEnabledInteractions() const { return mResetEnabledInteractions; }
  bool resetMaximalInteractions() const { return mResetMaximalInteractions; }
  bool resetLocallyMaximalInteractions() const { return mResetLocallyMaximalInteractions; }
  Interaction &allEnabledPorts() const;

  void computeIsTopLevel() const;

  bool mInitializedDependentConnectors;
  vector<Connector *> mDependentConnectors;

  bool mInitializedDependentPriorities;
  vector<Priority *> mDependentPriorities;

  mutable bool mIsReset;

  mutable bool mResetAllEnabledPorts;
  mutable Interaction *mAllEnabledPorts;

  mutable bool mInitializedInteractions;
  mutable vector<Interaction *> mInteractions;

  mutable bool mResetEnabledInteractions;
  mutable vector<InteractionValue *> mEnabledInteractions;

  mutable bool mResetMaximalInteractions;
  mutable vector<InteractionValue *> mMaximalInteractions;

  mutable bool mResetLocallyMaximalInteractions;
  mutable vector<InteractionValue *> mLocallyMaximalInteractions;

  mutable bool mInitializedDominatingPriorities;
  mutable vector<Priority *> mDominatingPriorities;

  mutable bool mInitializedDominatedPriorities;
  mutable vector<Priority *> mDominatedPriorities;

  mutable bool mInitializedIsLowLevel;
  mutable bool mIsLowLevel;

  mutable bool mInitializedIsTopLevel;
  mutable bool mIsTopLevel;

  mutable bool mInitializedSubConnectors;
  mutable vector<Connector *> mSubConnectors;

  mutable bool mInitializedAllSubConnectors;
  mutable vector<Connector *> mAllSubConnectors;
};

inline bool Connector::isTopLevel() const {
  if (!mInitializedIsTopLevel) {
    computeIsTopLevel();
  }

  return mIsTopLevel;
}

#endif // _BIP_Engine_Connector_HPP_
